/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

import {useCallback, useEffect, useRef} from 'react';

const RECT_ATTRS = ['bottom', 'height', 'left', 'right', 'top', 'width'];

const EMPTY_OBJECT = {};

const rectChanged = (a, b = EMPTY_OBJECT) =>
	RECT_ATTRS.some((prop) => a[prop] !== b[prop]);

export function useObserveRect(callback, node) {
	const rafIdRef = useRef();

	const run = useCallback(
		(node, state) => {
			const newRect = node.getBoundingClientRect();

			if (rectChanged(newRect, state.rect)) {
				state.rect = newRect;

				callback(state.rect);
			}

			rafIdRef.current = window.requestAnimationFrame(() =>
				run(node, state)
			);
		},
		[callback]
	);

	useEffect(() => {
		if (node) {
			run(node, {
				hasRectChanged: false,
				rect: undefined,
			});

			return () => {
				if (rafIdRef.current) {
					cancelAnimationFrame(rafIdRef.current);
				}
			};
		}
	}, [node, run]);

	if (!node) {
		return;
	}
}
